-----------------------------------------------------------------------
--
--  File:        adagraph.adb
--  Description: basic Win32 graphics
--  Rev:         0.5c
--  Date:        23-jan-1999
--  Author:      Jerry van Dijk
--  Mail:        jdijk@acm.org
--
--  Copyright (c) Jerry van Dijk, 1997, 1998, 1999
--  Billie Hollidaystraat 28
--  2324 LK Leiden
--  THE NETHERLANDS
--  tel int +31 (0)71 531 4365
--
--  Permission granted to use for any purpose, provided this copyright
--  remains attached and unmodified.
--
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--
-----------------------------------------------------------------------

with Interfaces.C;

package body AdaGraph is

   pragma Linker_Options ("-ladagraph");

   --------------------------------
   -- Make the C types available --
   --------------------------------

   package C renames Interfaces.C;

   ----------------------------
   -- DLL Internal constants --
   ----------------------------

   No_Errors        : constant Integer := 0;
   Default_Window   : constant Integer := 0;
   MouseNone        : constant Integer := 0;
   MouseMove        : constant Integer := 1;
   Maximized_Window : constant Integer := 1;
   MouseLeftUp      : constant Integer := 2;
   MouseRightUp     : constant Integer := 3;
   MouseLeftDown    : constant Integer := 4;
   MouseRightDown   : constant Integer := 5;

   ------------------------
   -- DLL Internal types --
   ------------------------

   type MouseEventStruct is
      record
         Event : Integer;
         Xpos  : Integer;
         Ypos  : Integer;
      end record;
   pragma Convention (C, MouseEventStruct);

   type MouseEventStructAccess is access all MouseEventStruct;
   pragma Convention (C, MouseEventStructAccess);

   type Integer_Access is access all Integer;
   pragma Convention (C, Integer_Access);

   ------------------------------
   -- Import the DLL functions --
   ------------------------------

   function MouseEvent return Integer;
   pragma Import (C, MouseEvent, "MouseEvent");

   function GetMouse (Event : MouseEventStructAccess) return Integer;
   pragma Import (C, GetMouse, "GetMouse");

   function GetKey return Integer;
   pragma Import (C, GetKey, "GetKey");

   function KeyHit return Integer;
   pragma Import (C, KeyHit, "KeyHit");

   function IsOpen return Integer;
   pragma Import (C, IsOpen, "IsOpen");

   function GetFontWidth return Integer;
   pragma Import (C, GetFontWidth, "GetFontWidth");

   function GetFontHeight return Integer;
   pragma Import (C, GetFontHeight, "GetFontHeight");

   function GetDllVersion return Integer;
   pragma Import (C, GetDllVersion, "GetDLLVersion");

   function GetWindowWidth return Integer;
   pragma Import (C, GetWindowWidth, "GetWindowWidth");

   function GetWindowHeight return Integer;
   pragma Import (C, GetWindowHeight, "GetWindowHeight");

   function CreateGraphWindow (Size : Integer) return Integer;
   pragma Import (C, CreateGraphWindow, "CreateGraphWindow");

   function CreateSizedGraphWindow (X, Y : Integer) return Integer;
   pragma Import (C, CreateSizedGraphWindow, "CreateSizedGraphWindow");

   function DestroyGraphWindow return Integer;
   pragma Import (C, DestroyGraphWindow, "DestroyGraphWindow");

   function ClearWindow (Hue : Integer) return Integer;
   pragma Import (C, ClearWindow, "ClearWindow");

   function GetColorPixel (X, Y: Integer) return Integer;
   pragma Import (C, GetColorPixel, "GetColorPixel");

   function Putpixel (X, Y, Hue : Integer) return Integer;
   pragma Import (C, Putpixel, "PutPixel");

   function FillFlood (X, Y, Hue : Integer) return Integer;
   pragma Import (C, FillFlood, "FillFlood");

   function SetWindowTitle (Title : C.Char_Array) return Integer;
   pragma Import (C, SetWindowTitle, "SetWindowTitle");

   function DrawLine (X1, Y1, X2, Y2, Hue : Integer) return Integer;
   pragma Import (C, DrawLine, "DrawLine");

   function DrawBox (X1, Y1, X2, Y2, Hue, Filled : Integer) return Integer;
   pragma Import (C, DrawBox, "DrawBox");

   function DrawCircle (X, Y, Radius, Hue, Filled : Integer) return Integer;
   pragma Import (C, DrawCircle, "DrawCircle");

   function DrawEllipse (X1, Y1, X2, Y2, Hue, Filled : Integer) return Integer;
   pragma Import (C, DrawEllipse, "DrawEllipse");

   function DisplayText (X, Y : Integer; Text : C.Char_Array; Hue : Integer) return Integer;
   pragma Import (C, DisplayText, "DisplayText");

   function GetMaxSize (X, Y : Integer_Access) return Integer;
   pragma Import (C, GetMaxSize, "GetMaxSize");

   function WhereX return Integer;
   pragma Import (C, WhereX, "WhereX");

   function WhereY return Integer;
   pragma Import (C, WhereY, "WhereY");

   function GotoXY (X, Y : Integer) return Integer;
   pragma Import (C, GotoXY, "GotoXY");

   function DrawTo (X, Y, Hue : Integer) return Integer;
   pragma Import (C, DrawTo, "DrawTo");

   ---------------------------------------------------
   -- Translate DLL error codes into Ada exceptions --
   ---------------------------------------------------

   procedure Make_Exception (Error : in Integer) is
   begin
      if Error < No_Errors then
         case Error is
            when  -1    => raise Window_Already_Open;
            when  -2    => raise Window_Already_Closed;
            when  -3    => raise Create_Event_Failed;
            when  -4    => raise Create_Thread_Failed;
            when  -5    => raise Window_Not_Open;
            when  -6    => raise Invalid_Color_Value;
            when  -7    => raise Invalid_Coordinate;
            when  -8    => raise Error_Copying_Title;
            when  -9    => raise Error_Copying_Cmdline;
            when -10    => raise Wait_Failed_Error;
            when -11    => raise Set_Title_Error;
            when -12    => raise Fill_Rect_Error;
            when -13    => raise Invalidate_Rect_Error;
            when -14    => raise Update_Window_Error;
            when -15    => raise Set_Pixel_Error;
            when -16    => raise Select_Pen_Error;
            when -17    => raise Move_To_Error;
            when -18    => raise Line_To_Error;
            when -19    => raise Select_Brush_Error;
            when -20    => raise Rectangle_Error;
            when -21    => raise Ellipse_Error;
            when -22    => raise Get_Pixel_Error;
            when -23    => raise Flood_Fill_Error;
            when -24    => raise Set_Textcolor_Error;
            when -25    => raise Text_Out_Error;
            when -26    => raise Invalid_Window_Size;
            when -27    => raise Get_Position_Error;
            when -28    => raise Close_Handle_Failed;
            when -29    => raise Thread_Status_Error;
            when others => raise Unknown_Adagraph_Error;
         end case;
      end if;
   end Make_Exception;

   ----------------------------
   -- Call the DLL functions --
   ----------------------------

   --------------------------------------------------------------------
   function Is_Open return Boolean is
      Result : Boolean;
   begin
      case IsOpen is
         when 0      => Result := False;
         when 1      => Result := True;
         when others => raise Unknown_Adagraph_Error;
      end case;
      return Result;
   end Is_Open;

   --------------------------------------------------------------------
   function Get_Dll_Version return Integer is
      Return_Value : Integer;
   begin
      Return_Value := GetDllVersion;
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      return Return_Value;
   end Get_Dll_Version;

   --------------------------------------------------------------------
   procedure Get_Max_Size (X_Size, Y_Size : out Integer) is
      X, Y         : aliased Integer;
      Return_Value : Integer;
      X_Access     : Integer_Access := X'Unchecked_Access;
      Y_Access     : Integer_Access := Y'Unchecked_Access;
   begin
      Return_Value := GetMaxSize (X_Access, Y_Access);
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      X_Size := X;
      Y_Size := Y;
   end Get_Max_Size;

   --------------------------------------------------------------------
   procedure Create_Graph_Window (X_Max,  Y_Max  : out Integer;
                                  X_Char, Y_Char : out Integer) is
      Return_Value : Integer;
   begin
      Return_Value := CreateGraphWindow (Default_Window);
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      X_Max := Integer (GetWindowWidth);
      Y_Max := Integer (GetWindowHeight);
      if (X_Max = 0) or (Y_Max = 0) then
         raise Error_Reading_Size;
      end if;
      X_Char := Integer (GetFontWidth);
      Y_Char := Integer (GetFontHeight);
      if (X_Char = 0) or (Y_Char = 0) then
         raise Error_Reading_Font;
      end if;
   end Create_Graph_Window;

   --------------------------------------------------------------------
   procedure Ext_Create_Graph_Window (X_Max,  Y_Max  : out Integer;
                                      X_Char, Y_Char : out Integer) is
      Return_Value : Integer;
   begin
      Return_Value := CreateGraphWindow(Maximized_Window);
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      X_Max := Integer (GetWindowWidth);
      Y_Max := Integer (GetWindowHeight);
      if X_Max = 0 or Y_Max = 0 then
         raise Error_Reading_Size;
      end if;
      X_Char := Integer (GetFontWidth);
      Y_Char := Integer (GetFontHeight);
      if X_Char = 0 or Y_Char = 0 then
         raise Error_Reading_Font;
      end if;
   end Ext_Create_Graph_Window;

   --------------------------------------------------------------------
   procedure Create_Sized_Graph_Window (X_Size, Y_Size : in     Integer;
                                        X_Max,  Y_Max  :    out Integer;
                                        X_Char, Y_Char :    out Integer) is
      Return_Value : Integer;
   begin
      Return_Value := CreateSizedGraphWindow (X_Size, Y_Size);
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      X_Max := Integer (GetWindowWidth);
      Y_Max := Integer (GetWindowHeight);
      if X_Max = 0 or Y_Max = 0 then
         raise Error_Reading_Size;
      end if;
      X_Char := Integer (GetFontWidth);
      Y_Char := Integer (GetFontHeight);
      if X_Char = 0 or Y_Char = 0 then
         raise Error_Reading_Font;
      end if;
   end Create_Sized_Graph_Window;

   --------------------------------------------------------------------
   procedure Destroy_Graph_Window is
      Return_Value : Integer;
   begin
      Return_Value := DestroyGraphWindow;
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Destroy_Graph_Window;

   --------------------------------------------------------------------
   procedure Clear_Window (Hue : in Color_Type := Black) is
      Return_Value : Integer;
   begin
      Return_Value := ClearWindow (Color_Type'Pos (Hue));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Clear_Window;

   --------------------------------------------------------------------
   procedure Set_Window_Title(Title : in String) is
      Return_Value : Integer;
   begin
      Return_Value := SetWindowTitle (C.To_C (Title));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Set_Window_Title;

   --------------------------------------------------------------------
   function Get_Pixel (X, Y : Integer) return Color_Type is
      Return_Value : Integer;
   begin
      Return_Value := GetColorPixel (X, Y);
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      return Color_Type'Val (Return_Value);
   end Get_Pixel;

   --------------------------------------------------------------------
   procedure Put_Pixel (X, Y : in Integer; Hue : in Color_Type := White) is
      Return_Value : Integer;
   begin
      Return_Value := PutPixel (X, Y, Color_Type'Pos (Hue));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Put_Pixel;

   --------------------------------------------------------------------
   function Key_Hit return Boolean is
      Return_Value : Integer;
   begin
      Return_Value := KeyHit;
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      return Return_Value /= 0;
   end Key_Hit;

   --------------------------------------------------------------------
   function Get_Key return Character is
      Return_Value : Integer;
   begin
      Return_Value := GetKey;
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      return Character'Val (Return_Value);
   end Get_Key;

   --------------------------------------------------------------------
   function Mouse_Event return Boolean is
      Return_Value : Integer;
   begin
      Return_Value := MouseEvent;
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      return Return_Value /= 0;
   end Mouse_Event;

   --------------------------------------------------------------------
   function Get_Mouse return Mouse_Type is
      Return_Value : Integer;
      Result       : Mouse_Type;
      The_Mouse    : aliased MouseEventStruct;
      Our_Mouse    : MouseEventStructAccess := The_Mouse'Unchecked_Access;
   begin
      Return_Value := GetMouse (Our_Mouse);
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      if The_Mouse.Event = MouseNone then
         Result.X_Pos := 0;
         Result.Y_Pos := 0;
      else
         Result.X_Pos := The_Mouse.Xpos;
         Result.Y_Pos := The_Mouse.Ypos;
      end if;
      case The_Mouse.Event is
         when MouseNone      => Result.Event := None;
         when MouseMove      => Result.Event := Moved;
         when MouseLeftUp    => Result.Event := Left_Up;
         when MouseRightUp   => Result.Event := Right_Up;
         when MouseLeftDown  => Result.Event := Left_Down;
         when MouseRightDown => Result.Event := Right_Down;
         when others         => raise Mouse_Event_Error;
      end case;
      return Result;
   end Get_Mouse;

   --------------------------------------------------------------------
   procedure Draw_Line (X1, Y1, X2, Y2 : in Integer;
                        Hue            : in Color_Type := White) is
      Return_Value : Integer;
   begin
      Return_Value := DrawLine (X1, Y1, X2, Y2, Color_Type'Pos (Hue));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Draw_Line;

   --------------------------------------------------------------------
   procedure Draw_Box (X1, Y1, X2, Y2 : in Integer;
                       Hue            : in Color_Type := White;
                       Filled         : in Fill_Type  := No_Fill) is
      Return_Value : Integer;
   begin
      Return_Value := DrawBox (X1, Y1, X2, Y2, Color_Type'Pos (Hue), Fill_Type'Pos (Filled));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Draw_Box;

   --------------------------------------------------------------------
   procedure Draw_Circle (X, Y, Radius : in Integer;
                          Hue          : in Color_Type := White;
                          Filled       : in Fill_Type  := No_Fill) is
      Return_Value : Integer;
   begin
      Return_Value := DrawCircle (X, Y, Radius, Color_Type'Pos (Hue), Fill_Type'Pos (Filled));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Draw_Circle;

   --------------------------------------------------------------------
   procedure Draw_Ellipse (X1, Y1, X2, Y2 : in Integer;
                           Hue            : in Color_Type := White;
                           Filled         : in Fill_Type  := No_Fill) is
      Return_Value : Integer;
   begin
      Return_Value := DrawEllipse (X1, Y1, X2, Y2, Color_Type'Pos (Hue), Fill_Type'Pos (Filled));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Draw_Ellipse;

   --------------------------------------------------------------------
   procedure Flood_Fill (X, Y : in Integer; Hue : in Color_Type := White) is
      Return_Value : Integer;
   begin
      Return_Value := FillFlood (X, Y, Color_Type'Pos (Hue));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Flood_Fill;

   --------------------------------------------------------------------
   procedure Display_Text (X, Y : in Integer;
                           Text : in String;
                           Hue  : in Color_Type := White) is
      Return_Value : Integer;
   begin
      Return_Value := DisplayText (X, Y, C.To_C (Text), Color_Type'Pos (Hue));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Display_Text;

   --------------------------------------------------------------------
   function Where_X return Integer is
      Return_Value : Integer;
   begin
      Return_Value := WhereX;
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      return Return_Value;
   end Where_X;

   --------------------------------------------------------------------
   function Where_Y return Integer is
      Return_Value : Integer;
   begin
      Return_Value := WhereY;
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
      return Return_Value;
   end Where_Y;

   --------------------------------------------------------------------
   procedure Goto_XY (X, Y : in Integer) is
      Return_Value : Integer;
   begin
      Return_Value := GotoXY (X, Y);
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Goto_XY;

   --------------------------------------------------------------------
   procedure Draw_To (X, Y : in Integer; Hue : in Color_Type := White) is
      Return_Value : Integer;
   begin
      Return_Value := DrawTo (X, Y, Color_Type'Pos (Hue));
      if Return_Value < No_Errors then
         Make_Exception (Return_Value);
      end if;
   end Draw_To;

-------------------------------------------------------
-- Check that the right version of the DLL is loaded --
-------------------------------------------------------
begin
   if Get_Dll_Version < Adagraph_Dll_Version then
      raise Dll_Version_Error;
   end if;
end Adagraph;
